IOStatWrapper

Struct IOStatWrapper 

pub struct IOStatWrapper<T, C> { /* private fields */ }
A wrapper around an IO object that tracks operations and statistics.

Implementations

Source

impl<T, C> IOStatWrapper<T, C>

pub fn new(obj: T, start_seek_pos: u64) -> IOStatWrapper<T, C>

Create a new IOStatWrapper with a manually given seek position. Detecting the seek position automatically is not possible without specialization.

Examples found in repository?
examples/bufread_reduced_read_count.rs (line 8)
6fn main() {
7    let file_obj = File::open("Cargo.toml").unwrap();
8    let mut instrumented_raw_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(file_obj, 0);
9    let buffered_io = BufReader::new(&mut instrumented_raw_file);
10    let mut instrumented_buf_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(buffered_io, 0);
11
12    // Do something so that the loop doesn't get optimized out
13    let mut xor_result: u8 = 0x00;
14    for byte in (&mut instrumented_buf_file).bytes() {
15        xor_result ^= byte.unwrap();
16    }
17    println!("XOR of all bytes in Cargo.toml is {:#x}", xor_result);
18
19    // Demonstrate how BufReader reduces the number of read calls
20    println!("Buffered read was called successfully {} times",
21        instrumented_buf_file.read_call_counter().success_ctr());
22    println!("Inner read was called successfully {} times",
23        instrumented_raw_file.read_call_counter().success_ctr());
24}
More examples
Hide additional examples
examples/bufread_assert_stream_position.rs (line 8)
6fn main() {
7    let file_obj = File::open("Cargo.toml").unwrap();
8    let mut instrumented_raw_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(file_obj, 0);
9    let buffered_io = BufReader::new(&mut instrumented_raw_file);
10    let mut instrumented_buf_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(buffered_io, 0);
11
12    // Do something so that the loop doesn't get optimized out
13    let mut xor_result: u8 = 0x00;
14    let mut read_buf: [u8; 1] = [0x00; 1];
15    loop {
16        let bytes_read = instrumented_buf_file.read(&mut read_buf).unwrap();
17        if bytes_read == 0 {
18            break;
19        }
20        xor_result ^= read_buf[0];
21        assert_eq!(instrumented_buf_file.seek_pos(),
22            instrumented_buf_file.stream_position().unwrap());
23    }
24    println!("XOR of all bytes in Cargo.toml is {:#x}", xor_result);
25
26    // Demonstrate how BufReader reduces the number of read calls
27    println!("Buffered read was called successfully {} times",
28        instrumented_buf_file.read_call_counter().success_ctr());
29    println!("Buffered seek was called successfully {} times",
30        instrumented_buf_file.seek_call_counter().success_ctr());
31    println!("Inner read was called successfully {} times",
32        instrumented_raw_file.read_call_counter().success_ctr());
33    println!("Inner seek was called successfully {} times",
34        instrumented_raw_file.seek_call_counter().success_ctr());
35}
examples/bufread_assert_seek_zero.rs (line 8)
6fn main() {
7    let file_obj = File::open("Cargo.toml").unwrap();
8    let mut instrumented_raw_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(file_obj, 0);
9    let buffered_io = BufReader::new(&mut instrumented_raw_file);
10    let mut instrumented_buf_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(buffered_io, 0);
11
12    // Do something so that the loop doesn't get optimized out
13    let mut xor_result: u8 = 0x00;
14    let mut read_buf: [u8; 1] = [0x00; 1];
15    loop {
16        let bytes_read = instrumented_buf_file.read(&mut read_buf).unwrap();
17        if bytes_read == 0 {
18            break;
19        }
20        xor_result ^= read_buf[0];
21        assert_eq!(instrumented_buf_file.seek_pos(),
22            instrumented_buf_file.seek(SeekFrom::Current(0)).unwrap());
23    }
24    println!("XOR of all bytes in Cargo.toml is {:#x}", xor_result);
25
26    // Demonstrate how BufReader reduces the number of read calls
27    println!("Buffered read was called successfully {} times",
28        instrumented_buf_file.read_call_counter().success_ctr());
29    println!("Buffered seek was called successfully {} times",
30        instrumented_buf_file.seek_call_counter().success_ctr());
31    println!("Inner read was called successfully {} times",
32        instrumented_raw_file.read_call_counter().success_ctr());
33    println!("Inner seek was called successfully {} times",
34        instrumented_raw_file.seek_call_counter().success_ctr());
35}
pub fn into_inner(self) -> T

Extract the original I/O object.

pub fn iop_log(&self) -> &C

Get the I/O operation log containing operations and their results.

Source

impl<T: Read, C> IOStatWrapper<T, C>

pub fn read_call_counter(&self) -> &SuccessFailureCounter<u64>

Returns the number of times Read::read() was invoked.

Examples found in repository?
examples/bufread_reduced_read_count.rs (line 21)
6fn main() {
7    let file_obj = File::open("Cargo.toml").unwrap();
8    let mut instrumented_raw_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(file_obj, 0);
9    let buffered_io = BufReader::new(&mut instrumented_raw_file);
10    let mut instrumented_buf_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(buffered_io, 0);
11
12    // Do something so that the loop doesn't get optimized out
13    let mut xor_result: u8 = 0x00;
14    for byte in (&mut instrumented_buf_file).bytes() {
15        xor_result ^= byte.unwrap();
16    }
17    println!("XOR of all bytes in Cargo.toml is {:#x}", xor_result);
18
19    // Demonstrate how BufReader reduces the number of read calls
20    println!("Buffered read was called successfully {} times",
21        instrumented_buf_file.read_call_counter().success_ctr());
22    println!("Inner read was called successfully {} times",
23        instrumented_raw_file.read_call_counter().success_ctr());
24}
More examples
Hide additional examples
examples/bufread_assert_stream_position.rs (line 28)
6fn main() {
7    let file_obj = File::open("Cargo.toml").unwrap();
8    let mut instrumented_raw_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(file_obj, 0);
9    let buffered_io = BufReader::new(&mut instrumented_raw_file);
10    let mut instrumented_buf_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(buffered_io, 0);
11
12    // Do something so that the loop doesn't get optimized out
13    let mut xor_result: u8 = 0x00;
14    let mut read_buf: [u8; 1] = [0x00; 1];
15    loop {
16        let bytes_read = instrumented_buf_file.read(&mut read_buf).unwrap();
17        if bytes_read == 0 {
18            break;
19        }
20        xor_result ^= read_buf[0];
21        assert_eq!(instrumented_buf_file.seek_pos(),
22            instrumented_buf_file.stream_position().unwrap());
23    }
24    println!("XOR of all bytes in Cargo.toml is {:#x}", xor_result);
25
26    // Demonstrate how BufReader reduces the number of read calls
27    println!("Buffered read was called successfully {} times",
28        instrumented_buf_file.read_call_counter().success_ctr());
29    println!("Buffered seek was called successfully {} times",
30        instrumented_buf_file.seek_call_counter().success_ctr());
31    println!("Inner read was called successfully {} times",
32        instrumented_raw_file.read_call_counter().success_ctr());
33    println!("Inner seek was called successfully {} times",
34        instrumented_raw_file.seek_call_counter().success_ctr());
35}
examples/bufread_assert_seek_zero.rs (line 28)
6fn main() {
7    let file_obj = File::open("Cargo.toml").unwrap();
8    let mut instrumented_raw_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(file_obj, 0);
9    let buffered_io = BufReader::new(&mut instrumented_raw_file);
10    let mut instrumented_buf_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(buffered_io, 0);
11
12    // Do something so that the loop doesn't get optimized out
13    let mut xor_result: u8 = 0x00;
14    let mut read_buf: [u8; 1] = [0x00; 1];
15    loop {
16        let bytes_read = instrumented_buf_file.read(&mut read_buf).unwrap();
17        if bytes_read == 0 {
18            break;
19        }
20        xor_result ^= read_buf[0];
21        assert_eq!(instrumented_buf_file.seek_pos(),
22            instrumented_buf_file.seek(SeekFrom::Current(0)).unwrap());
23    }
24    println!("XOR of all bytes in Cargo.toml is {:#x}", xor_result);
25
26    // Demonstrate how BufReader reduces the number of read calls
27    println!("Buffered read was called successfully {} times",
28        instrumented_buf_file.read_call_counter().success_ctr());
29    println!("Buffered seek was called successfully {} times",
30        instrumented_buf_file.seek_call_counter().success_ctr());
31    println!("Inner read was called successfully {} times",
32        instrumented_raw_file.read_call_counter().success_ctr());
33    println!("Inner seek was called successfully {} times",
34        instrumented_raw_file.seek_call_counter().success_ctr());
35}
pub fn read_byte_counter(&self) -> usize

Returns the total number of bytes read.

Source

impl<T: Seek, C> IOStatWrapper<T, C>

pub fn seek_call_counter(&self) -> &SuccessFailureCounter<u64>

Returns the number of times Seek::seek() was invoked.

Examples found in repository?
examples/bufread_assert_stream_position.rs (line 30)
6fn main() {
7    let file_obj = File::open("Cargo.toml").unwrap();
8    let mut instrumented_raw_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(file_obj, 0);
9    let buffered_io = BufReader::new(&mut instrumented_raw_file);
10    let mut instrumented_buf_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(buffered_io, 0);
11
12    // Do something so that the loop doesn't get optimized out
13    let mut xor_result: u8 = 0x00;
14    let mut read_buf: [u8; 1] = [0x00; 1];
15    loop {
16        let bytes_read = instrumented_buf_file.read(&mut read_buf).unwrap();
17        if bytes_read == 0 {
18            break;
19        }
20        xor_result ^= read_buf[0];
21        assert_eq!(instrumented_buf_file.seek_pos(),
22            instrumented_buf_file.stream_position().unwrap());
23    }
24    println!("XOR of all bytes in Cargo.toml is {:#x}", xor_result);
25
26    // Demonstrate how BufReader reduces the number of read calls
27    println!("Buffered read was called successfully {} times",
28        instrumented_buf_file.read_call_counter().success_ctr());
29    println!("Buffered seek was called successfully {} times",
30        instrumented_buf_file.seek_call_counter().success_ctr());
31    println!("Inner read was called successfully {} times",
32        instrumented_raw_file.read_call_counter().success_ctr());
33    println!("Inner seek was called successfully {} times",
34        instrumented_raw_file.seek_call_counter().success_ctr());
35}
More examples
Hide additional examples
examples/bufread_assert_seek_zero.rs (line 30)
6fn main() {
7    let file_obj = File::open("Cargo.toml").unwrap();
8    let mut instrumented_raw_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(file_obj, 0);
9    let buffered_io = BufReader::new(&mut instrumented_raw_file);
10    let mut instrumented_buf_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(buffered_io, 0);
11
12    // Do something so that the loop doesn't get optimized out
13    let mut xor_result: u8 = 0x00;
14    let mut read_buf: [u8; 1] = [0x00; 1];
15    loop {
16        let bytes_read = instrumented_buf_file.read(&mut read_buf).unwrap();
17        if bytes_read == 0 {
18            break;
19        }
20        xor_result ^= read_buf[0];
21        assert_eq!(instrumented_buf_file.seek_pos(),
22            instrumented_buf_file.seek(SeekFrom::Current(0)).unwrap());
23    }
24    println!("XOR of all bytes in Cargo.toml is {:#x}", xor_result);
25
26    // Demonstrate how BufReader reduces the number of read calls
27    println!("Buffered read was called successfully {} times",
28        instrumented_buf_file.read_call_counter().success_ctr());
29    println!("Buffered seek was called successfully {} times",
30        instrumented_buf_file.seek_call_counter().success_ctr());
31    println!("Inner read was called successfully {} times",
32        instrumented_raw_file.read_call_counter().success_ctr());
33    println!("Inner seek was called successfully {} times",
34        instrumented_raw_file.seek_call_counter().success_ctr());
35}
pub fn seek_pos(&self) -> u64

Get the current seek position without doing an actual seek operation.

This is accomplished by storing a separate position integer. When debug assertions are on we assert after every seek operation that the cursor is where we expect it to be.

Examples found in repository?
examples/bufread_assert_stream_position.rs (line 21)
6fn main() {
7    let file_obj = File::open("Cargo.toml").unwrap();
8    let mut instrumented_raw_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(file_obj, 0);
9    let buffered_io = BufReader::new(&mut instrumented_raw_file);
10    let mut instrumented_buf_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(buffered_io, 0);
11
12    // Do something so that the loop doesn't get optimized out
13    let mut xor_result: u8 = 0x00;
14    let mut read_buf: [u8; 1] = [0x00; 1];
15    loop {
16        let bytes_read = instrumented_buf_file.read(&mut read_buf).unwrap();
17        if bytes_read == 0 {
18            break;
19        }
20        xor_result ^= read_buf[0];
21        assert_eq!(instrumented_buf_file.seek_pos(),
22            instrumented_buf_file.stream_position().unwrap());
23    }
24    println!("XOR of all bytes in Cargo.toml is {:#x}", xor_result);
25
26    // Demonstrate how BufReader reduces the number of read calls
27    println!("Buffered read was called successfully {} times",
28        instrumented_buf_file.read_call_counter().success_ctr());
29    println!("Buffered seek was called successfully {} times",
30        instrumented_buf_file.seek_call_counter().success_ctr());
31    println!("Inner read was called successfully {} times",
32        instrumented_raw_file.read_call_counter().success_ctr());
33    println!("Inner seek was called successfully {} times",
34        instrumented_raw_file.seek_call_counter().success_ctr());
35}
More examples
Hide additional examples
examples/bufread_assert_seek_zero.rs (line 21)
6fn main() {
7    let file_obj = File::open("Cargo.toml").unwrap();
8    let mut instrumented_raw_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(file_obj, 0);
9    let buffered_io = BufReader::new(&mut instrumented_raw_file);
10    let mut instrumented_buf_file = IOStatWrapper::<_, Vec<IopInfoPair>>::new(buffered_io, 0);
11
12    // Do something so that the loop doesn't get optimized out
13    let mut xor_result: u8 = 0x00;
14    let mut read_buf: [u8; 1] = [0x00; 1];
15    loop {
16        let bytes_read = instrumented_buf_file.read(&mut read_buf).unwrap();
17        if bytes_read == 0 {
18            break;
19        }
20        xor_result ^= read_buf[0];
21        assert_eq!(instrumented_buf_file.seek_pos(),
22            instrumented_buf_file.seek(SeekFrom::Current(0)).unwrap());
23    }
24    println!("XOR of all bytes in Cargo.toml is {:#x}", xor_result);
25
26    // Demonstrate how BufReader reduces the number of read calls
27    println!("Buffered read was called successfully {} times",
28        instrumented_buf_file.read_call_counter().success_ctr());
29    println!("Buffered seek was called successfully {} times",
30        instrumented_buf_file.seek_call_counter().success_ctr());
31    println!("Inner read was called successfully {} times",
32        instrumented_raw_file.read_call_counter().success_ctr());
33    println!("Inner seek was called successfully {} times",
34        instrumented_raw_file.seek_call_counter().success_ctr());
35}
Source

impl<T: Write, C> IOStatWrapper<T, C>

pub fn write_call_counter(&self) -> &SuccessFailureCounter<u64>

Returns the number of times Write::write() was invoked.

pub fn write_flush_counter(&self) -> &SuccessFailureCounter<u64>

Returns the number of times Write::flush() was invoked.

pub fn write_byte_counter(&self) -> usize